/*
 * Copyright (C) 2006  Chris Rose
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package com.aimedia.azjabber;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.gudy.azureus2.plugins.download.Download;

import com.aimedia.azjabber.config.IAzJabberMessageKeys;

public class StateTransition {
    private static Set<StateTransition> stateSet                = new HashSet<StateTransition> ();

    public static final StateTransition QUEUED_TO_WAITING       = new StateTransition (Download.ST_QUEUED,
                                                                        Download.ST_WAITING, "queued_to_waiting");

    public static final StateTransition WAITING_TO_PREPARING    = new StateTransition (Download.ST_WAITING,
                                                                        Download.ST_PREPARING, "waiting_to_preparing");

    public static final StateTransition WAITING_TO_WAITING      = new StateTransition (Download.ST_WAITING,
                                                                        Download.ST_WAITING, "waiting_to_waiting");

    public static final StateTransition PREPARING_TO_READY      = new StateTransition (Download.ST_PREPARING,
                                                                        Download.ST_READY, "preparing_to_ready");

    public static final StateTransition READY_TO_DOWNLOADING    = new StateTransition (Download.ST_READY,
                                                                        Download.ST_DOWNLOADING, "ready_to_downloading");

    public static final StateTransition READY_TO_SEEDING        = new StateTransition (Download.ST_READY,
                                                                        Download.ST_SEEDING, "ready_to_seeding");

    public static final StateTransition READY_TO_STOPPED        = new StateTransition (Download.ST_READY,
                                                                        Download.ST_STOPPED, "ready_to_stopped");

    public static final StateTransition DOWNLOADING_TO_SEEDING  = new StateTransition (Download.ST_DOWNLOADING,
                                                                        Download.ST_SEEDING, "downloading_to_seeding");

    public static final StateTransition DOWNLOADING_TO_STOPPING = new StateTransition (Download.ST_DOWNLOADING,
                                                                        Download.ST_STOPPING, "downloading_to_stopping");

    public static final StateTransition DOWNLOADING_TO_STOPPED  = new StateTransition (Download.ST_DOWNLOADING,
                                                                        Download.ST_STOPPED, "downloading_to_stopped");

    public static final StateTransition SEEDING_TO_STOPPING     = new StateTransition (Download.ST_SEEDING,
                                                                        Download.ST_STOPPING, "seeding_to_stopping");

    public static final StateTransition SEEDING_TO_STOPPED      = new StateTransition (Download.ST_SEEDING,
                                                                        Download.ST_STOPPED, "seeding_to_stopped");

    public static final StateTransition STOPPING_TO_STOPPED     = new StateTransition (Download.ST_STOPPING,
                                                                        Download.ST_STOPPED, "stopping_to_stopped");

    public static final StateTransition STOPPED_TO_QUEUED       = new StateTransition (Download.ST_STOPPED,
                                                                        Download.ST_QUEUED, "stopped_to_queued");

    private int                         oldStatus;

    private int                         newStatus;

    private String                      name;

    private StateTransition (int oldStatus, int newStatus, String name) {
        this (oldStatus, newStatus, true);

        this.name = name;
    }

    private StateTransition (int oldStatus, int newStatus, boolean addToSet) {
        this.oldStatus = oldStatus;
        this.newStatus = newStatus;
        
        this.name = "Unnamed transition from " + oldStatus + " to " + newStatus;

        if (addToSet) {
            stateSet.add (this);
        }
    }

    public static final StateTransition forStates (int oldState, int newState) {
        StateTransition key = new StateTransition (oldState, newState, false);
        StateTransition ret = key;

        for (Iterator<StateTransition> iter = stateSet.iterator (); iter.hasNext ();) {
            StateTransition t = iter.next ();

            if (t.equals (key)) {
                ret = t;
            }
        }
        return ret;
    }

    public boolean equals (Object o) {
        return this.oldStatus == ((StateTransition) o).oldStatus && this.newStatus == ((StateTransition) o).newStatus;
    }

    public int hashCode () {
        return oldStatus % (newStatus * 17);
    }

    public String getName () {
        return this.name;
    }

    public String toString () {
        return getName ();
    }
}